/* @(#)fetchrecord.c	1.4 9/4/92 */
/***************************************************************************/
/*
        Name: fetchrecord.c
 Description: Returns the item values of the first record which matches
                the values for the search items.  Output record is written
                to stdout in an AML readable form.  An optional variable
                prefix may be prefixed to all output AML vaiables.
     History: 01/92  Todd Stellhorn  - original coding
              04/92  Todd Stellhorn  - final coding
              04/22/92 Todd Stellhorn - fixed nul problem on output
              09/92  Todd Stellhorn - flush stdout
 */
/***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "infotools.h"

/***************************************************************************/

#define USAGE "usage: fetchrecord <INFO_File> <Item1> <Value1> <Itemn> <Valuen> {Variable_Prefix}"

typedef struct
{
     char              *SearchItemName;
     char              *SearchString;
     long int           SearchInt;
     short int          RecnoFlag;
     InfoItemDef       *SearchItemPtr;
} SearchItemList;

short int OutputRecord();

/***************************************************************************/

int main( argc, argv )

int argc;
char **argv;

/***************************************************************************/
/*
       Name: main
    Purpose: Main fetchrecord.c function.
      Usage: INFO_File - input INFO file name
             Search_Item1 - name of the first search item
             Search_Value1 - first search value
             Search_Itemn - name of the n'th search item
             Search_Valuen - n'th search value
             Variable_Prefix - AML global variable prefix
    Outputs: Function ( int ) - program status
             Returned record directed to stdout.
    History: 01/92  Todd Stellhorn  - original coding
             04/92  Todd Stellhorn  - final coding
 */
{
      char              *InfoFilePath;
      char               VarPrefix[128];
      short int          NumberKeys;
      SearchItemList    *SILHome;
      SearchItemList    *SILCurrent;
      short int          ArgNumber = 1;
      long int           RecnoFlag = 0;
      InfoFile          *FilePtr;
      char               OutputString[4096];
      long int           RecNum = 1;
      long int           i;
      short int          CompareCode;
      long int           RecordNumber = 0;

      if( argc < 3 )
           FatalError( USAGE );

      InfoFilePath = argv[ ArgNumber++ ];
           
      NumberKeys = ( argc - 2 ) / 2;

/* malloc for Search Item List */

      if(( SILHome = SILCurrent = ( SearchItemList * ) 
           malloc( sizeof( SearchItemList ) * NumberKeys )) == NULL )
           FatalError( "Memory allocation error." );
      
/* open file ... */

      if(( FilePtr = InfoOpenFile( InfoFilePath, InfoREAD )) == 
           ( InfoFile * ) NULL )
           FatalError( "Cannot open INFO file." );

/* fill SearchItemList ... */

      for( i = 0; i < NumberKeys; i++ )
      {
           SILCurrent->SearchItemName = argv[ ArgNumber++ ];
           SILCurrent->SearchString = argv[ ArgNumber++ ]; 
           SILCurrent->SearchInt = atoi( SILCurrent->SearchString );

           if(( SILCurrent->SearchItemPtr = 
                InfoGetItemDef( FilePtr, SILCurrent->SearchItemName )) 
                == ( InfoItemDef * ) NULL )
                FatalError( "Error getting item definition." );

           if( SILCurrent->SearchItemPtr->ItemType == INFO_RECNO_TYPE )
           {
                SILCurrent->RecnoFlag = 1;
                RecnoFlag = SILCurrent->SearchInt;
           }
           else
                SILCurrent->RecnoFlag = 0;

           SILCurrent++;
      }

/* test for variable prefix */

      VarPrefix[0] = '\0';

      if ((argc - 2) % 2) 
           (void) strcpy( VarPrefix, argv[argc-1] );

/* start at top of search item list */

      SILCurrent = SILHome;

      if( RecnoFlag )  /* if $RECNO has been input as a search item */
      {
           if( ! InfoReadRecord( FilePtr, RecnoFlag ))
                FatalError( "Error reading record." );

           for( i = 0; i < NumberKeys; i++ )
           {
                if( ! SILCurrent->RecnoFlag )
                {
                     if( ! CompareValues( FilePtr, SILCurrent->SearchItemPtr, 
                          SILCurrent->SearchString, SILCurrent->SearchInt, &CompareCode ))
                          FatalError( "Value compare error." );

                     /* if we made it here we can assume that $RECNO was specified on the 
                        command line.  if so, and the above CompareValues() call failed,
                        ie, CompareCode != 0, then a matching record cannot be found */
                        
                     if( CompareCode )
                          FatalError( "Matching value not found." );
                }

                SILCurrent++; /* next search item list record */
           }

           RecordNumber = FilePtr->CurrentRecord;

           if( ! OutputRecord( FilePtr, RecordNumber, VarPrefix ))
                FatalError( "Error outputing record." );

           goto DONE;
      }
      else  /* $RECNO has not been input as search item */
      {
           while( RecNum <= FilePtr->NumberRecords )
           {
                if( ! InfoReadRecord( FilePtr, RecNum++ ))
                     FatalError( "Error reading record." );

                SILCurrent = SILHome;  /* start at top */

                for( i = 0; i < NumberKeys; i++ )
                {
                     if( ! CompareValues( FilePtr, SILCurrent->SearchItemPtr, 
                          SILCurrent->SearchString, SILCurrent->SearchInt, &CompareCode ))
                          FatalError( "Value compare error." );

                     if( CompareCode ) /* non-matching record, so exit the for loop */
                          break;

                     SILCurrent++;  /* (else) next search item list record */
                }

                /* if CompareCode == 0, then we have found our record */

                if( ! CompareCode ) 
                {
                     RecordNumber = FilePtr->CurrentRecord;

                     if( ! OutputRecord( FilePtr, RecordNumber, VarPrefix ))
                          FatalError( "Error outputing record." );

                     goto DONE;
                }
           }
      }

      DONE:

/* close up shop */

      if( ! InfoCloseFile( FilePtr ))
           FatalError( "Error closing INFO file." );

      /* RecordNumber will be non 0 if a matching record has been found */

      if( RecordNumber )
      {
           (void) fprintf( stdout, OK_STRING );
           (void) fflush( stdout );
      }
      else
           FatalError( "Matching record not found." );

      return( 0 );
}                                           /* end of main                 */


/***************************************************************************/

short int OutputRecord( FilePtr, RecordNumber, VarPrefix )

InfoFile *FilePtr;
long int RecordNumber;
char *VarPrefix;

/***************************************************************************/
/*
       Name: OutputRecord
    Purpose: Outputs matched record to stdout.
     Inputs: FilePtr ( InfoFile * ) - INFO file
             RecordNumber ( long int ) - matching record number
             VarPrefix ( char * ) - output AML variable prefix
    Outputs: Function ( short int )- function status 
             Input INFO record to stdout.
    History: 01/92  Todd Stellhorn  - original coding
             04/92  Todd Stellhorn  - final coding
 */
{
      TemplateList      *Template;
      char               RetString[4096];
      double             DRetVal;
      char               TmpString[4096];
      char               OutReturnString[4096];
      short int          i;

/* get template */

      if(( Template = InfoGetTemplate( FilePtr )) == ( TemplateList * ) NULL )
           return( 0 );

/* read record */

      if( ! InfoReadRecord( FilePtr, RecordNumber ))
           return( 0 );

/* decode record and output */

      for( i = 0; i < FilePtr->NumberItems; i++ )
      {
           if( ! InfoDecode( FilePtr, &Template[i], RetString, &DRetVal ))
                return( 0 );

           (void) sprintf( OutReturnString, "&s %s.%s := ", VarPrefix, 
                Template[i].ItemName );
 
           switch( Template[i].ItemType )
           {
                case INFO_DATE_TYPE         :  
                     return( 0 );

                case INFO_CHARACTER_TYPE    :
                     CompressString( RetString, strlen( RetString ));
                     (void) sprintf( TmpString, "'%s'", RetString );
                     strcat( OutReturnString, TmpString );
                     break;

                case INFO_INTEGER_TYPE      :
                     (void) sprintf( TmpString, "%d", ( int ) DRetVal );
                     strcat( OutReturnString, TmpString );
                     break;

                case INFO_NUMBER_TYPE       : 
                     (void) sprintf( TmpString, "%f", DRetVal );
                     strcat( OutReturnString, TmpString );
                     break;

                case INFO_BINARY_TYPE       :
                     (void) sprintf( TmpString, "%d", ( int ) DRetVal );
                     strcat( OutReturnString, TmpString );
                     break;

                case INFO_FLOATING_TYPE     :
                     (void) sprintf( TmpString, "%f", DRetVal );
                     strcat( OutReturnString, TmpString );
                     break;

                case INFO_LEADFILL_TYPE     :
                case INFO_PACKED_TYPE       :
                case INFO_ZEROFILL_TYPE     :
                case INFO_OVERPUNCH_TYPE    :
                case INFO_TRAILINGSIGN_TYPE :
                case INFO_RECNO_TYPE        :
                default                     :
                     return( 0 );

           }

           (void) fprintf( stdout, "%s\n", OutReturnString );
      }

      return( 1 );  /* valid return */
}                                           /* end of OutputRecord         */

/*                                             end of fetchrecord.c        */
