/* @(#)loadinfo.c	1.3 9/4/92 */
/***************************************************************************/
/*
        Name: loadinfo.c
 Description: Loads data into an existing INFO template from a system 
                file.  The format for the system file is contained in an
                INFO file named <Template>.  The template file must have
                the following items:

                  POSITION - item record position
                  WIDTH - item record width
                  ITEM_NAME - item name
                
                System file is assumed to not contain newline characters.
     History: 01/92  Todd Stellhorn  - original coding
              04/92  Todd Stellhorn  - final coding
              05/92  Todd Stellhorn  - added missing valid return string
              09/92  Todd Stellhorn  - flush stdout
 */
/***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "infotools.h"

/***************************************************************************/

typedef struct
{
     short int          Position;
     short int          Width;
     char               ItemName[17];
     InfoItemDef       *ItemPtr;
} ItemList;

#define USAGE "usage: loadinfo -init <INFO_File> <Template> <In_File>"

/***************************************************************************/

int main( argc, argv )

int argc;
char **argv;

/***************************************************************************/
/*
       Name: main
    Purpose: Main loadinfo function.
     Inputs: -init - init flag -- will truncate INFO_File length to zero
                on file open
             INFO_File - subject INFO file
             Template - INFO template file
             In_File - input system file
    Outputs: Function ( int ) - function status
             On success, INFO_File is fill with data values from In_File.
             On error, contents of INFO_File is undefined.
    History: 01/92  Todd Stellhorn  - original coding
             04/92  Todd Stellhorn  - final coding
 */
{
      short int          ArgNumber = 1;
      short int          init_flag = 0;
      short int          RecordSize = 0;
      char              *LoadFilePath;
      char              *TemplateFilePath;
      char              *InFile;
      InfoFile          *LoadFilePtr;
      InfoFile          *TemplateFilePtr;
      FILE              *InFilePtr;
      ItemList          *ItemListPtr;
      ItemList          *TempItemListPtr;
      short int          NumTemplateRecords;
      InfoItemDef       *PositionDef;
      InfoItemDef       *WidthDef;
      InfoItemDef       *ItemNameDef;
      char              *ReadBuffer;
      char               RetString[4096];
      double             RetDouble;
      char               TempString[4096];
      double             TempDouble;
      long int           RecordCount = 0, i, k;
      char               TempMess[1024];

/* test for args */

      if( argc == 1 )
           FatalError( USAGE );

/* test for init flag */

      if( argv[ ArgNumber ][0] == '-' )
      {
           if( ! CompareStrings( argv[ ArgNumber++ ], "-init", NONCASE_SENSITIVE ))
                init_flag = 1;
           else
                FatalError( USAGE );
      }

/* collect rest of arguments */
      
      if(( argc - ArgNumber ) < 3 )
           FatalError( USAGE );

      LoadFilePath = argv[ ArgNumber++ ];
      TemplateFilePath = argv[ ArgNumber++ ];
      InFile = argv[ ArgNumber++ ];

/* open files */

      if( init_flag )
      {
           if(( LoadFilePtr = InfoOpenFile( LoadFilePath, InfoINIT )) == NULL )
           {  
                (void) sprintf( TempMess, "Cannot open INFO file %s.", LoadFilePath );
                FatalError( TempMess );
           }  
      }
      else
      {
           if(( LoadFilePtr = InfoOpenFile( LoadFilePath, InfoWRITE )) == NULL )
           {  
                (void) sprintf( TempMess, "Cannot open INFO file %s.", LoadFilePath );
                FatalError( TempMess );
           }  
      }

      if(( TemplateFilePtr = InfoOpenFile( TemplateFilePath, InfoREAD )) == NULL )
      {
           (void) sprintf( TempMess, "Cannot open INFO file %s.", TemplateFilePath );           
           FatalError( TempMess );
      }  

      if(( InFilePtr = fopen( InFile, "rb" )) == NULL )
      {  
           (void) sprintf( TempMess, "Cannot open data file %s.", InFile );
           FatalError( TempMess );
      }  

/* get item definitions */

      if(( PositionDef = InfoGetItemDef( TemplateFilePtr, "POSITION" )) == NULL )
      {
           (void) sprintf( TempMess, "Cannot find item %s on INFO file %s.",
                "POSITION", TemplateFilePtr->FileName );
           FatalError( TempMess );
      }  

      if(( WidthDef = InfoGetItemDef( TemplateFilePtr, "WIDTH" )) == NULL )
      {  
           (void) sprintf( TempMess, "Cannot find item %s on INFO file %s.",
                "WIDTH", TemplateFilePtr->FileName );
           FatalError( TempMess );
      }  

      if(( ItemNameDef = InfoGetItemDef( TemplateFilePtr, "ITEM_NAME" )) == NULL )
      {
           (void) sprintf( TempMess, "Cannot find item %s on INFO file %s.",
                "ITEM_NAME", TemplateFilePtr->FileName );
           FatalError( TempMess );
      }  

/* malloc space for itemlist array */

      if(( ItemListPtr = TempItemListPtr = ( ItemList * ) 
           malloc(( sizeof( ItemList ) * TemplateFilePtr->NumberRecords ))) == NULL )
           FatalError( "Memory allocation error." );

/* fill itemlist array */

      NumTemplateRecords = TemplateFilePtr->NumberRecords;

      for( i = 1; i <= NumTemplateRecords; i++ )
      {
           if( ! InfoReadRecord( TemplateFilePtr, i ))
           {
                (void) sprintf( TempMess, "Error: InfoReadRecord, %s at record: %ld", 
                     TemplateFilePtr->FileName, i ); 
                FatalError( TempMess );
           }

           if( ! InfoDecode( TemplateFilePtr, PositionDef, RetString, &RetDouble ))
           {
                (void) sprintf( TempMess, "Error: InfoDecode; %s at item %s.",
                     TemplateFilePtr->FileName, PositionDef->ItemName );
                FatalError( TempMess );
           }
           TempItemListPtr->Position = ( short int ) RetDouble;

           if( ! InfoDecode( TemplateFilePtr, WidthDef, RetString, &RetDouble ))
           {
                (void) sprintf( TempMess, "Error: InfoDecode; %s at item %s.",
                     TemplateFilePtr->FileName, WidthDef->ItemName );
                FatalError( TempMess );
           }
           TempItemListPtr->Width = ( short int ) RetDouble;
           RecordSize = RecordSize + TempItemListPtr->Width;

           if( ! InfoDecode( TemplateFilePtr, ItemNameDef, RetString, &RetDouble ))
           {
                (void) sprintf( TempMess, "Error: InfoDecode; %s at item %s.",
                     TemplateFilePtr->FileName, ItemNameDef->ItemName );
                FatalError( TempMess );
           }
           CompressString( RetString, ( sizeof( RetString )));
           (void) strcpy( TempItemListPtr->ItemName, RetString );
           CompressString( TempItemListPtr->ItemName, ( sizeof( TempItemListPtr->ItemName )));

           if( TempItemListPtr->ItemName[0] != '*' )
           {
                if(( TempItemListPtr->ItemPtr = 
                     InfoGetItemDef( LoadFilePtr, TempItemListPtr->ItemName )) == NULL )
                {
                     (void) sprintf( TempMess, "Cannot find item %s on INFO file %s.", 
                          TempItemListPtr->ItemName, LoadFilePtr->FileName );
                     FatalError( TempMess );
                }
           }
           else
                TempItemListPtr->ItemPtr = ( InfoItemDef * ) NULL;

           TempItemListPtr++;
      }

      if( ! InfoCloseFile( TemplateFilePtr ))
      {
           (void) sprintf( TempMess, "Cannot close INFO file %s.", TemplateFilePath );
           FatalError( TempMess );
      }

      if( ! RecordSize )
           FatalError( "Input record size of zero." );

/* init readbuffer */

      if(( ReadBuffer = ( char * ) malloc( sizeof( char ) * RecordSize )) == NULL )
           FatalError( "Memory allocation error." );

/* foreach record in infile... */

      while( fread( ReadBuffer, ( sizeof( char ) * RecordSize ), 1, InFilePtr ))
      {
           RecordCount++;
           TempItemListPtr = ItemListPtr;

/* if writting to existing file, first read record, else init buffer */

           if( RecordCount <= LoadFilePtr->NumberRecords )
           {
                if( ! InfoReadRecord( LoadFilePtr, RecordCount ))
                {
                     (void) sprintf( TempMess, "Error: InfoReadRecord, %s at record: %ld", 
                          LoadFilePtr->FileName, RecordCount ); 
                     FatalError( TempMess );
                }
                else
                     (void) memset( LoadFilePtr->IOBuffer, 32, LoadFilePtr->RecordLength );
           }

/* foreach item from the template */

           for( i = 0; i < NumTemplateRecords; i++ )
           {
                (void) strncpy( TempString, &ReadBuffer[ TempItemListPtr->Position - 1 ], 
                     TempItemListPtr->Width );
                TempString[ TempItemListPtr->Width ] = '\0';
                TempDouble = ( double ) atof( TempString );

                if( ! InfoEncode( LoadFilePtr, TempItemListPtr->ItemPtr, 
                     TempString, TempDouble ))
                {
                     (void) sprintf( TempMess, "Error: InfoEncode; %s at item %s.",
                          LoadFilePtr->FileName, TempItemListPtr->ItemPtr->ItemName );
                     FatalError( TempMess );
                }

                TempItemListPtr++;
           }
          
           if( ! InfoWriteRecord( LoadFilePtr, RecordCount ))
           {
                (void) sprintf( TempMess, "Error: InfoWriteRecord, %s at record: %ld", 
                     LoadFilePtr->FileName, RecordCount ); 
                FatalError( TempMess );
           }
      }

      if( ! InfoCloseFile( LoadFilePtr ))
      {
           (void) sprintf( TempMess, "Cannot close INFO file %s.", LoadFilePath );
           FatalError( TempMess );
      }

      (void) fclose( InFilePtr );

      (void) fprintf( stdout, OK_STRING );
      (void) fflush( stdout );

      return( 0 );
}                                           /* end of main                 */

/*                                             end of loadinfo.c           */
